// CountWordsCmd.cc

#include <stdio.h>
#include <math.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "CountWordsCmd.h"
#include <InfoDialogManager.h>

#define NUMLINES 50	// try to read this many on each call

CountWordsCmd::CountWordsCmd ( char *name, int active, char *filename ) : BreakCmd ( name, active )
{
	struct stat statInfo;
	
	// initialize data members
	_bytesRead = 0;
	_fileSize = 0;
	_percentDone = 0;
	
	// open file
	if ( ( _fd = fopen ( filename, "r" ) ) == NULL )
	{
		char buf[BUFSIZE];
		
		sprintf ( buf, "Can't open %s", filename );
		infoDialogManager->post ( buf );
	}
	else
	{
		// check file size as basis for progress report
		char buf[BUFSIZE];
		
		if ( stat ( filename, &statInfo ) == 0 )
		{
			_fileSize = statInfo.st_size;
			// empirically determined for particular program
			updateTimeout ( ( unsigned long ) 500 );
		}	
		
		if ( _fileSize == 0 )
		{
			sprintf ( buf, "%s is empty!", filename );
			infoDialogManager->post ( buf );
			fclose ( _fd );
			_fd = 0;
		}
	}
}


CountWordsCmd::~CountWordsCmd()
{
	if ( _fd )
		fclose ( _fd );
}
	


void CountWordsCmd::doCommand()
{
	char buf[BUFSIZE];
	char *sep = " !@#$%^&*()_+=-}{}{|\';:\"?></.,`~\\\n\t";
	int percent;
	
	if ( !_fd )
	{
		_done = TRUE;
		return;
	}

	// read a few lines when doCommand is called
	for ( int i = 0; i < NUMLINES; i++ )
	{
		char *result;
		char *word;
		
		//read in one line; indicate done if EOF
		
		if ( ( result = fgets ( buf, BUFSIZE, _fd ) ) == NULL )
		{
			_done = TRUE;
			return;
		}
		
		// get total chars read for progress report
		_bytesRead += strlen ( buf );
		
		// extract first full word and save in word list
		word = strtok ( buf, sep );
		saveWord ( word );
		
		// continue extraction until line end
		while ( ( word = strtok ( NULL, sep ) ) != NULL )
			saveWord ( word );
	}
	
	percent = ( int ) ( ( float ) _bytesRead / ( float ) _fileSize * 100 );

	if ( _percentDone != percent )
	{
		_percentDone = percent;
		sprintf ( buf, "%d%%", _percentDone );		
		updateMessage ( buf );
	}
}


void CountWordsCmd::saveWord ( char *word )
{
	if ( !word )
		return;
	
	for ( int i = 0; i < _list.Count(); i++ )
	{
		if ( strcmp ( _list[i]->word(), word ) == 0 )
		{
			_list[i]->incrementCount();
			return;
		}
	}
	
	// List class tries to avoid pointers to pointers by using a reference
	Word *temp = new Word ( word );

	// if word not found, create new Word object, add it to list
	_list.Add ( temp );
}
